/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.fidentis.analyst.feature.services;

import cz.fidentis.analyst.feature.FeaturePoint;
import cz.fidentis.analyst.feature.provider.FeaturePointTypeProvider;
import cz.fidentis.analyst.mesh.io.MeshObjLoader;
import java.io.File;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.Optional;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.junit.jupiter.api.Assertions.assertTrue;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.DisplayName;

/**
 *
 * @author Jakub Kolman
 */
public class FeaturePointImportServiceTest {
    
    
    private static final Path testFileDirectory = Paths.get("src", "test", "resources", "cz", "fidentis", "analyst", "feature", "services");
    private static final String TEST_CSV_FILE = "00007_01_landmarks.csv";
    private static final String TEST_FP_FILE = "fp_head1_test.fp";
     
    @Test()
    void fileExistsTest() throws IOException {
        File testCsv = new File(testFileDirectory.toFile(), TEST_CSV_FILE);
        assertTrue(testCsv.canRead());
    }
    
    @DisplayName("Test loading a CSV file")
    @Test
    void importFeaturePointsCSVTest() {
        Optional<List<FeaturePoint>> featurePoints = loadFeaturePoints();
        assertTrue(featurePoints.isPresent());
        assertTrue(featurePoints.get().size() > 0);
        assertTrue(featurePoints.get().get(0).getX() == -45.3298
                && featurePoints.get().get(0).getY() == 37.1466
                && featurePoints.get().get(0).getZ() == -40.5415
                && featurePoints.get().get(0).getFeaturePointType().equals(
                        FeaturePointTypeProvider.getInstance().getFeaturePointTypeByCode("EX_R"))
        );
    }

    private Optional<List<FeaturePoint>> loadFeaturePoints() {
        FeaturePointsCsvLoader service = new FeaturePointsCsvLoader();
        Optional<List<FeaturePoint>> featurePoints = service.importFeaturePointsCSV(testFileDirectory.toString(), TEST_CSV_FILE);
        return featurePoints;
    }
    
    @DisplayName("Test loading a FP file")
    @Test
    void importFeaturePointsFPTest() {
        FeaturePointsFpLoader service = new FeaturePointsFpLoader();
        Optional<List<FeaturePoint>> featurePoints = service.importFeaturePointsFP(testFileDirectory.toString(), TEST_FP_FILE);
        assertTrue(featurePoints.isPresent());
        assertTrue(featurePoints.get().size() > 0);
        assertTrue(featurePoints.get().get(0).getX() == -40.429775
                && featurePoints.get().get(0).getY() == 45.756405
                && featurePoints.get().get(0).getZ() == 37.12462
                && featurePoints.get().get(0).getFeaturePointType().equals(
                        FeaturePointTypeProvider.getInstance().getFeaturePointTypeByCode("EX_R"))
        );
    }
}
